@@ -47,6 +47,10 @@ See file COPYING for restrictions on the use of this software.
 
 #include "groupselect.h"
 
+#ifndef min
+#define min(a,b) ((a < b) ? (a) : (b))
+#endif
+
 /*
  * misc. global variables, documented in leafnode.h
  */
@@ -86,46 +90,72 @@ int allow_8bit_headers = 0;
 char *newsadmin;
 unsigned long timeout_lock = 5UL;
 
-static int parse_line( /*@unique@*/ char *l, /*@out@*/ char *param,	/*@out@*/
-		      char *value);
+/** parse the line in \a l, breaking it into param and value at the "="
+ * delimiter. The right-hand side can be quoted with double quotes,
+ * inside these a backslash escapes a quote that is part of the string.
+ * \return success
+ */
+static int parse_line(
+	/*@unique@*/ char *l /** input, will be modified */,
+	/*@out@*/ char *param /** output, left-hand side */,
+	/*@out@*/ char *value /** output, right-hand side */);
 
 /* parse a line, destructively */
-int
+static int
 parse_line(char *l, char *param, char *value)
 {
-    char *p, *q;
+    char *p;
+    size_t le, len;
+    enum modes { plain, quoted } mode = plain;
 
     p = l;
-    /* skip comments */
-    q = strchr(p, '#');
-    if (q)
-	*q = '\0';
-    if (*p == '\0')
-	return 0;
-    if ((q = strchr(p, '=')) == NULL)
-	return 0;
-    else
-	*q++ = '\0';
-    /* skipping leading blanks or tabs */
+    /* skip leading spaces, read parameter */
     SKIPLWS(p);
-    SKIPLWS(q);
-    (void)strlcpy(param, p, TOKENSIZE);
-    (void)strlcpy(value, q, TOKENSIZE);
+    le = strcspn(p, "=#");
+    /* strip trailing space */
+    while(le && strchr(" \t", p[le-1])) le--;
+    len = min(le, TOKENSIZE - 1);
+    if (!len) return 0;
+    memcpy(param, p, len);
+    param[len] = '\0';
+    p += le;
 
-    /* now param contains the stuff before '=' and value the stuff behind it */
-    if (!(*param) || !(*value))
+    SKIPLWS(p);
+    if (*p++ != '=')
 	return 0;
-    /* skipping trailing blanks or tabs */
-    p = param + strlen(param) - 1;
-    q = value + strlen(value) - 1;
-    while (isspace((unsigned char)*p) && (p > param)) {
-	*p = '\0';
-	p--;
+    SKIPLWS(p);
+
+    /* strip trailing blanks from input */
+    le = strlen(p);
+    while (le--) {
+	if (p[le] == ' ' || p[le] == '\t')
+	    p[le] = '\0';
+	else
+	    break;
     }
-    while (isspace((unsigned char)*q) && (q > value)) {
-	*q = '\0';
-	q--;
+
+    /* read value */
+    for (le = 0 ; le < TOKENSIZE - 1 ; le ++) {
+	char c = *p++;
+	if (mode == plain) {
+	    if (c == '#' || c == '\0') { break; }
+	    if (c == '"') { mode = quoted; continue; }
+	    *value++ = c;
+	} else if (mode == quoted) {
+	    if (c == '\\') {
+		if (*p) {
+		    *value++ = *p++; continue; 
+		} else
+		    return 0;
+	    }
+	    if (c == '\0') return 0;
+	    if (c == '"') break;
+	    *value++ = c;
+	} else {
+	    abort();
+	}
     }
+    *value = '\0';
     return 1;
 }
 
@@ -154,8 +184,8 @@ static int read_timeout_lock(
 	if (debugmode) {
 	    if (timeout_lock) {
 		syslog(LOG_DEBUG,
-			"%s: waiting %lu seconds for lockfile",
-			source, timeout_lock);
+			"%s: waiting %lu second%s for lockfile",
+			source, timeout_lock, PLURAL(timeout_lock));
 	    } else {
 		syslog(LOG_DEBUG,
 			"%s: waiting indefinitely for lockfile", source);
@@ -218,8 +248,8 @@ readconfig(int logtostderr)
 		if (p) {
 		    p->timeout = atoi(value);
 		    if (debugmode)
-			syslog(LOG_DEBUG, "config: timeout is %d seconds",
-			       p->timeout);
+			syslog(LOG_DEBUG, "config: timeout is %d second%s",
+			       p->timeout, PLURAL(p->timeout));
 		} else
 		    syslog(LOG_ERR, "config: no server for timeout");
 	    } else if (strcmp("allowSTRANGERS", param) == 0) {
@@ -248,7 +278,7 @@ readconfig(int logtostderr)
 		expiredays = i;
 		expire = time(NULL) - (time_t) (SECONDS_PER_DAY * i);
 		if (debugmode)
-		    syslog(LOG_DEBUG, "config: expire is %d days", i);
+		    syslog(LOG_DEBUG, "config: expire is %d day%s", i, PLURAL(i));
 	    } else if (strcmp("newsadmin", param) == 0) {
 		if (debugmode)
 		    syslog(LOG_DEBUG, "config: newsadmin is %s", value);
@@ -274,8 +304,8 @@ readconfig(int logtostderr)
 		/* maxgroups is for compatibility with leafnode+ */
 		crosspostlimit = strtol(value, NULL, 10);
 		if (debugmode)
-		    syslog(LOG_DEBUG, "config: crosspostlimit is %ld groups",
-			   crosspostlimit);
+		    syslog(LOG_DEBUG, "config: crosspostlimit is %ld group%s",
+			   crosspostlimit, PLURAL(crosspostlimit));
 	    } else if (strcmp("article_despite_filter", param) == 0) {
 		article_despite_filter = atoi(value) ? 1 : 0;
 		if (debugmode)
@@ -289,18 +319,19 @@ readconfig(int logtostderr)
 	    } else if (strcmp("maxlines", param) == 0) {
 		maxlines = strtol(value, NULL, 10);
 		if (debugmode)
-		    syslog(LOG_DEBUG, "config: postings have max. %ld lines",
-			   maxlines);
+		    syslog(LOG_DEBUG, "config: postings have max. %ld line%s",
+			   maxlines, PLURAL(maxlines));
 	    } else if (strcmp("minlines", param) == 0) {
 		minlines = strtol(value, NULL, 10);
 		if (debugmode)
-		    syslog(LOG_DEBUG, "config: postings have min. %ld lines",
-			   minlines);
+		    syslog(LOG_DEBUG, "config: postings have min. %ld line%s",
+			   minlines, PLURAL(minlines));
 	    } else if (strcmp("maxbytes", param) == 0) {
 		maxbytes = strtoul(value, NULL, 10);
 		if (debugmode)
 		    syslog(LOG_DEBUG,
-			   "config: postings have max. %lu bytes", maxbytes);
+			   "config: postings have max. %lu byte%s",
+			   maxbytes, PLURAL(maxbytes));
 	    } else if (strcmp("linebuffer", param) == 0) {
 		linebuffer = atoi(value);
 		if (debugmode)
@@ -329,28 +360,28 @@ readconfig(int logtostderr)
 	    } else if (strcmp("timeout_short", param) == 0) {
 		timeout_short = atoi(value);
 		if (debugmode)
-		    syslog(LOG_DEBUG, "config: timeout_short is %d days",
-			   timeout_short);
+		    syslog(LOG_DEBUG, "config: timeout_short is %d day%s",
+			   timeout_short, PLURAL(timeout_short));
 	    } else if (strcmp("timeout_long", param) == 0) {
 		timeout_long = atoi(value);
 		if (debugmode)
-		    syslog(LOG_DEBUG, "config: timeout_long is %d days",
-			   timeout_long);
+		    syslog(LOG_DEBUG, "config: timeout_long is %d day%s",
+			   timeout_long, PLURAL(timeout_long));
 	    } else if (strcmp("timeout_active", param) == 0) {
 		timeout_active = atoi(value);
 		if (debugmode)
-		    syslog(LOG_DEBUG, "config: timeout_active is %d days",
-			   timeout_active);
+		    syslog(LOG_DEBUG, "config: timeout_active is %d day%s",
+			   timeout_active, PLURAL(timeout_active));
 	    } else if (strcmp("timeout_client", param) == 0) {
 		timeout_client = atoi(value);
 		if (debugmode)
-		    syslog(LOG_DEBUG, "config: timeout_client is %d secs",
-			   timeout_client);
+		    syslog(LOG_DEBUG, "config: timeout_client is %d second%s",
+			   timeout_client, PLURAL(timeout_client));
 	    } else if (strcmp("timeout_fetchnews", param) == 0) {
 		timeout_fetchnews = atoi(value);
 		if (debugmode)
-		    syslog(LOG_DEBUG, "config: timeout_fetchnews is %d secs",
-			   timeout_fetchnews);
+		    syslog(LOG_DEBUG, "config: timeout_fetchnews is %d second%s",
+			   timeout_fetchnews, PLURAL(timeout_fetchnews));
 	    } else if (strcmp("timeout_lock", param) == 0) {
 		read_timeout_lock(value, "config");
 	    } else if (strncmp("groupexpire", param, 11) == 0) {
@@ -377,8 +408,8 @@ readconfig(int logtostderr)
 			       m);
 			} else
 			    syslog(LOG_DEBUG,
-			           "config: groupexpire for %s is %ld days",
-			           m, (long)i);
+			           "config: groupexpire for %s is %ld day%s",
+			           m, (long)i, PLURAL(i));
 		    }
 		    e = time(NULL) - (time_t) (SECONDS_PER_DAY * i);
 		    ent = (struct expire_entry *)
