@@ -397,12 +397,21 @@ getbody_newno(const struct server *current_server, struct newsgroup *group, unsi
     xref = NULL;
     debug--;
     while ((l = getaline(f)) != NULL) {
-	if (!newsgroups && !strncmp(l, "Newsgroups:", 11))
-	    newsgroups = critstrdup(l + 11, "getbody");
-	if (!messageid && !strncmp(l, "Message-ID:", 11))
-	    messageid = critstrdup(l + 11, "getbody");
-	if (!xref && !strncmp(l, "Xref:", 5))
-	    xref = critstrdup(l + 5, "getbody");
+	if (!newsgroups && !strncmp(l, "Newsgroups:", 11)) {
+	    p = l+11;
+	    SKIPLWS(p);
+	    newsgroups = critstrdup(p, "getbody");
+	}
+	if (!messageid && !strncmp(l, "Message-ID:", 11)) {
+	    p = l+11;
+	    SKIPLWS(p);
+	    messageid = critstrdup(p, "getbody");
+	}
+	if (!xref && !strncmp(l, "Xref:", 5)) {
+	    p = l + 5;
+	    SKIPLWS(p);
+	    xref = critstrdup(p, "getbody");
+	}
     }
     debug = debugmode;
     fclose(f);
@@ -422,7 +431,7 @@ getbody_newno(const struct server *current_server, struct newsgroup *group, unsi
 
     xsnprintf(s, SIZE_s, "%lu", id);
     c = lookup(messageid);
-    unlink(c);			/* make space for new file */
+    log_unlink(c, 0);			/* make space for new file */
 
     if (!(f = fopen(c, "w"))) {
 	ln_log(LNLOG_SERR, LNLOG_CGROUP, "%s: cannot open %s for writing", group->name, c);
@@ -976,9 +985,9 @@ getgroup(const struct server *current_server,
 	return last + 1;
     }
 
-    syslog(LOG_INFO, "%s: will fetch %ld articles", g->name, outstanding);
+    syslog(LOG_INFO, "%s: will fetch %ld article%s", g->name, outstanding, PLURAL(outstanding));
     if (verbose > 1)
-	printf("%s: will fetch %ld articles\n", g->name, outstanding);
+	printf("%s: will fetch %ld article%s\n", g->name, outstanding, PLURAL(outstanding));
 
     if (minlines || maxlines) {
 	xsnprintf(lineout, SIZE_lineout, "XHDR Lines %lu-%lu\r\n", server,
@@ -1000,11 +1009,11 @@ getgroup(const struct server *current_server,
 		    if ((minlines && lines < minlines)
 			    || (maxlines && lines > maxlines)) {
 		    stufftoget[j] = 0;
-		    syslog(LOG_INFO, "%s: Killed article %lu: %ld lines",
-			   g->name, art, lines);
+		    syslog(LOG_INFO, "%s: Killed article %lu: %ld line%s",
+			   g->name, art, lines, PLURAL(lines));
 		    if (verbose > 2)
-			printf("%s: Killed article %lu: %ld lines.\n",
-			       g->name, art, lines);
+			printf("%s: Killed article %lu: %ld line%s.\n",
+			       g->name, art, lines, PLURAL(lines));
 		    killed++;
 		}
 	    }
@@ -1035,10 +1044,10 @@ getgroup(const struct server *current_server,
 		}
 		if (j < outstanding && (bytes > maxbytes)) {
 		    stufftoget[j] = 0;
-		    syslog(LOG_INFO, "%s: Killed article %lu (%lu > %lu) bytes",
+		    syslog(LOG_INFO, "%s: Killed article %lu (%lu > %lu bytes)",
 			   g->name, art, bytes, maxbytes);
 		    if (verbose > 2)
-			printf("%s: Killed article %lu (%lu > %lu) bytes.\n",
+			printf("%s: Killed article %lu (%lu > %lu bytes).\n",
 			       g->name, art, bytes, maxbytes);
 		    killed++;
 		}
@@ -1197,7 +1206,10 @@ getgroup(const struct server *current_server,
 	 *
 	 * SourceForge bug 873149, reported 2004-01-08 by Toni Viemer,
 	 * sourceforge user "skithund" */
-	if (strcmp(l, ".") == 0 && requested_body) {
+	if (l == NULL) {
+	    /* timeout - don't flush body */
+	    requested_body = FALSE;
+	} else if (strcmp(l, ".") == 0 && requested_body) {
 	    ln_log(LNLOG_SNOTICE, LNLOG_CTOP, "%s: %s:%lu: article without blank line after header, format violation",
 		    current_server->name, g->name, stufftoget[i]);
 	    requested_body = FALSE;
@@ -1242,10 +1254,10 @@ getgroup(const struct server *current_server,
 
 	if (localmaxage && age(hd[5]) > localmaxage) {
 	    if (verbose > 2)
-		printf("Discarding article %lu - older than %d days\n",
-		       stufftoget[i], localmaxage);
-	    syslog(LOG_INFO, "Discarding article %lu %s - older than %d days",
-		   stufftoget[i], hd[4], localmaxage);
+		printf("Discarding article %lu - older than %d day%s\n",
+		       stufftoget[i], localmaxage, PLURAL(localmaxage));
+	    syslog(LOG_INFO, "Discarding article %lu %s - older than %d day%s",
+		   stufftoget[i], hd[4], localmaxage, PLURAL(localmaxage));
 	    killed++;
 	    if (requested_body) ignore_answer(nntpin);
 	    continue;
@@ -1387,10 +1399,10 @@ getgroup(const struct server *current_server,
 		(void)ftruncate(fileno(f), 0);
 		(void)fclose(f);
 		unlink(c);
-		free(stufftoget);
 		ln_log(LNLOG_SWARNING, LNLOG_CARTICLE,
 			"warning: %s: %s: server disconnect or timeout after BODY %lu",
 			current_server->name, g->name, stufftoget[i]);
+		free(stufftoget);
 		return 0;
 	    }
 	    if (sscanf(l, "%3ld", &n) != 1 || (n / 10 != 22)) {
@@ -1428,19 +1440,19 @@ getgroup(const struct server *current_server,
 	if (l == NULL) {	/* article didn't terminate with a .: error */
 	    (void)truncate(c, 0);
 	    (void)unlink(c);
-	    free(stufftoget);
 		ln_log(LNLOG_SWARNING, LNLOG_CARTICLE,
 			"warning: %s: %s: server disconnect or timeout retrieving article %lu",
 			current_server->name, g->name, stufftoget[i]);
+	    free(stufftoget);
 	    return 0;
 	}
     }
 
-    syslog(LOG_INFO, "%s: %lu articles fetched (to %lu), %lu killed",
-	   g->name, fetched, g->last, killed);
+    syslog(LOG_INFO, "%s: %lu article%s fetched (to %lu), %lu killed",
+	   g->name, fetched, PLURAL(fetched), g->last, killed);
     if (verbose > 1)
-	printf("%s: %lu articles fetched, %lu killed\n",
-	       g->name, fetched, killed);
+	printf("%s: %lu article%s fetched, %lu killed\n",
+	       g->name, fetched, PLURAL(fetched), killed);
     free(stufftoget);
     return last + 1;
 }
@@ -1633,7 +1645,7 @@ nntpactive(struct server *current_server, time_t *stamp)
 	    ln_log(LNLOG_SERR, LNLOG_CSERVER,
 		    "%s: reading all newsgroups failed, reason \"%s\".",
 		    current_server->name, e);
-	    return -1;
+	    return -2;
 	}
 	debug--;
 	while ((l = mgetaline(nntpin)) && (strcmp(l, "."))) {
@@ -1653,7 +1665,7 @@ nntpactive(struct server *current_server, time_t *stamp)
 	    ln_log(LNLOG_SERR, LNLOG_CSERVER,
 		    "%s: reading all newsgroups failed, server disconnect or timeout.",
 		    current_server->name);
-	    return -1;
+	    return -2;
 	}
 	if (current_server->descriptions) {
 	    if (verbose)
@@ -1680,13 +1692,13 @@ nntpactive(struct server *current_server, time_t *stamp)
 			    current_server->name, l);
 		    ln_log(LNLOG_SERR, LNLOG_CSERVER,
 			    "Workaround: Add \"nodesc = 1\" (without quotes) below the server = %s line.", current_server->name);
-		    return -1;
+		    return -2;
 		}
 	    } else {
 		ln_log(LNLOG_SERR, LNLOG_CSERVER, 
 			"%s: reading newsgroups descriptions failed: server disconnect or timeout.",
 			current_server->name);
-		return -1;
+		return -2;
 	    }
 	    while (l && (strcmp(l, "."))) {
 		p = l;
@@ -1703,10 +1715,23 @@ nntpactive(struct server *current_server, time_t *stamp)
 		ln_log(LNLOG_SERR, LNLOG_CSERVER,
 			"%s: reading newsgroup descriptions failed, server disconnect or timeout.",
 			current_server->name);
-		return -1;
+		return -2;
 	    }
 	}
 	debug = debugmode;
+
+	/* mark active for /this/ server fetched */
+	{
+	    FILE *f = fopen(s, "a");
+	    if (!f) {
+		ln_log(LNLOG_SERR, LNLOG_CGROUP,
+			"cannot open \"%s\": %m", s);
+	    } else {
+		if (fclose(f))
+		    ln_log(LNLOG_SERR, LNLOG_CGROUP,
+			    "cannot close \"%s\": %m", s);
+	    }
+	}
     }
     *stamp = t;
     return 0;
@@ -1939,8 +1964,8 @@ free_ret:
     } /* while de = readdir */
     closedir(d);
     if (verbose)
-	printf("%s: %d articles posted.\n", current_server->name, n);
-    syslog(LOG_INFO, "%s: %d articles posted.", current_server->name, n);
+	printf("%s: %d article%s posted.\n", current_server->name, n, PLURAL(n));
+    syslog(LOG_INFO, "%s: %d article%s posted.", current_server->name, n, PLURAL(n));
     fchdir(savedir);
     close(savedir);
     return 1;
@@ -2097,7 +2122,8 @@ processupstream(const struct server *serv, time_t stamp)
 		    break;
 		}
 	    } else {
-		printf("%s not found in groupinfo file\n", de->d_name);
+		if (verbose > 1)
+		    printf("%s not found in groupinfo file\n", de->d_name);
 		syslog(LOG_NOTICE, "%s not found in groupinfo file", de->d_name);
 	    }
 	} /* if isalnum */
@@ -2216,6 +2242,7 @@ main(int argc, char **argv)
     volatile int rc = 0, skip_servers = 0;
     volatile int anypost = 0, waitchild = 0, quiet;
     struct server *current_server;
+    volatile int need_refetch = 0;
 
     int option, reply;
     pid_t pid;
@@ -2371,8 +2398,10 @@ main(int argc, char **argv)
 	    nntpquit();
 	    if (!rc)
 		rc = 2;
-	    if (forceactive)
+	    if (forceactive) {
 		error_refetch("caught signal that caused a premature abort.");
+		need_refetch = 1;
+	    }
 	    skip_servers = 1;	/* in this case, jump the while ... loop */
 	} else {
 	    canjump = 1;
@@ -2448,9 +2477,12 @@ main(int argc, char **argv)
 
 		    /* get list of newsgroups or new newsgroups */
 		    if (current_server->updateactive) {
-			if (nntpactive(current_server, &stamp)) {
-			    if (forceactive)
+			int r;
+			if ((r = nntpactive(current_server, &stamp))) {
+			    if (forceactive || r == -2) {
 				error_refetch("obtaining the active file failed.");
+				need_refetch = 1;
+			    }
 			    rc = 1;
 			}
 		    } else {
@@ -2468,8 +2500,10 @@ main(int argc, char **argv)
 	} else { /* reply = nntpconnect */
 	    if (verbose)
 		printf("%s: connection failed.\n", current_server->name);
-	    if (forceactive)
+	    if (forceactive && current_server->updateactive) {
 		error_refetch("fetching the active list from a server failed.");
+		need_refetch = 1;
+	    }
 	    rc = 2;
 	}
 	if (!usesupplement)
@@ -2531,16 +2565,22 @@ main(int argc, char **argv)
 	    /* only update active.read when have read active from all servers */
 	    if (writeactive()) {
 		ln_log(LNLOG_SERR, LNLOG_CTOP, "Error writing groupinfo.");
-		if (killactiveread()) {
-		    error_refetch("cannot write groupinfo file.");
+		error_refetch("cannot write groupinfo file.");
+		rc = 1;
+		/* mark for refetch */
+		killactiveread();
+	    } else {
+		/* active written successfully */
+		if (updateactive()) {
+		    error_refetch("cannot update active.read file.");
+		    need_refetch = 1;
 		    rc = 1;
 		}
-	    } else {
-		if (rc == 0 && updateactive())
-		    if (killactiveread()) {
-			error_refetch("cannot update active.read file.");
-			rc = 1;
-		    }
+
+		if (need_refetch) {
+		    /* mark for refetch */
+		    killactiveread();
+		}
 	    }
 
 #ifdef HAVE_WORKING_FORK
