@@ -3,7 +3,7 @@
  * conditions.
  */
 
-/* $Id: leafnode.h,v 1.90 2004/11/03 21:57:37 emma Exp $ */
+/* $Id: leafnode.h,v 1.92 2005/04/05 15:56:08 emma Exp $ */
 
 #ifndef LEAFNODE_H
 #define LEAFNODE_H
@@ -18,6 +18,8 @@
 #define FALSE (0)
 #endif
 
+#define PLURAL(no) (((no) == 1) ? "" : "s")
+
 /* limits.h may contain PATH_MAX but shall not if the system has variable
  * length limits here */
 #include <limits.h>
@@ -57,7 +59,7 @@ char *strdup(const char *);
 /* from snprintf.c */
 #ifndef HAVE_WORKING_SNPRINTF
 int ln_snprintf(char *str, size_t n, const char *format, ...)
-#ifdef __GNUC_
+#ifdef __GNUC__
     __attribute__ ((format(printf, 3, 4)))
 #endif
     ;
