@@ -316,14 +316,21 @@ static int check_linlist(const char *s, const struct versions *list, int count)
 int
 nntpconnect(const struct server *upstream)
 {
-    struct hostent *hp;
     static /*@observer@*/ struct servent *sp;
     struct servent sp_def;
+#ifdef HAVE_IPV6
+    struct addrinfo hints, *ai;
+    struct addrinfo *volatile aii;
+    char buf[INET6_ADDRSTRLEN+1];
+#else
     struct sockaddr_in s_in;
-    int sock, reply;
-    static int i;
+    struct hostent *hp;
+    char buf[16];
+    volatile int i;
+#endif
+    int sock, reply, e, ds;
+    char *line;
 
-    memset((void *)&s_in, 0, sizeof(s_in));
     if (upstream->port == 0) {
 	sp = getservbyname("nntp", "tcp");
 	if (sp == NULL) {
@@ -335,41 +342,79 @@ nntpconnect(const struct server *upstream)
 	sp->s_port = htons(upstream->port);
     }
 
+    sprintf(buf, "%hu", ntohs(sp->s_port));
+
     /* Fetch the ip addresses of the given host. */
+#ifdef HAVE_IPV6
+    memset((void *)&hints, 0, sizeof(hints));
+    hints.ai_family = AF_UNSPEC;
+    hints.ai_socktype = SOCK_STREAM;
+    hints.ai_protocol = IPPROTO_TCP;
+    hints.ai_flags = AI_CANONNAME|AI_ADDRCONFIG;
+
+    e = getaddrinfo(upstream->name, buf, &hints, &ai);
+    if (e == 0) {
+	for (aii = ai; aii; aii = aii->ai_next) {
+	    sock = socket(aii->ai_family, SOCK_STREAM, 0);
+	    if (sock < 0) {
+		ln_log(LNLOG_SERR, LNLOG_CTOP,
+			"error: cannot create inet/stream socket: %m");
+		break;
+	    }
+	    switch(aii->ai_family) {
+		case AF_INET6:
+		    inet_ntop(aii->ai_family, &((const struct sockaddr_in6 *)aii->ai_addr)->sin6_addr, buf, sizeof(buf));
+		    break;
+		case AF_INET:
+		    inet_ntop(aii->ai_family, &((const struct sockaddr_in *)aii->ai_addr)->sin_addr, buf, sizeof(buf));
+		    break;
+		default:
+		    strcpy(buf, "UNKNOWN");
+	    }
+#else
     hp = gethostbyname(upstream->name);
     if (hp) {
-	char *line;
-
 	/* Try to make connection to each of the addresses in turn. */
 	for (i = 0; (int *)(hp->h_addr_list)[i]; i++) {
-	    int e, ds;
-
-	    s_in.sin_family = hp->h_addrtype;
-	    s_in.sin_port = sp->s_port;
-	    s_in.sin_addr = incopy(hp->h_addr_list[i]);
-
 	    sock = socket(AF_INET, SOCK_STREAM, 0);
 	    if (sock < 0) {
 		ln_log(LNLOG_SERR, LNLOG_CTOP,
 			"error: cannot create inet/stream socket: %m");
 		break;
 	    }
+	    strcpy(buf, inet_ntoa(s_in.sin_addr));
+#endif
 
 	    if (sigsetjmp(timeout,1) != 0) {
 		ln_log(LNLOG_SWARNING, LNLOG_CTOP,
 			"warning: %s: connection to %s timed out",
-			upstream->name, inet_ntoa(s_in.sin_addr));
+			upstream->name, buf);
 		(void)close(sock);
 		continue;
 	    }
 
 	    (void)mysigact(SIGALRM, SA_RESETHAND, timer, 0);
 	    (void)alarm((unsigned)upstream->timeout);
+
+#ifdef HAVE_IPV6
+	    e = connect(sock, aii->ai_addr, aii->ai_addrlen);
+	    if (e) {
+		ln_log(LNLOG_SWARNING, LNLOG_CTOP,
+			"warning: %s: connection to %s failed: %m",
+			upstream->name, buf);
+	    }
+#else
+	    memset((void *)&s_in, 0, sizeof(s_in));
+	    s_in.sin_family = hp->h_addrtype;
+	    s_in.sin_port = sp->s_port;
+	    s_in.sin_addr = incopy(hp->h_addr_list[i]);
+
 	    e = connect(sock, (struct sockaddr *)&s_in, sizeof(s_in));
 	    if (e)
 		ln_log(LNLOG_SWARNING, LNLOG_CTOP,
 			"warning: %s: connection to %s failed: %m", 
 			upstream->name, inet_ntoa(s_in.sin_addr));
+#endif
 	    (void)alarm(0U);
 	    (void)mysigact(SIGALRM, 0, SIG_DFL, 0);
 	    if (e)
@@ -400,10 +445,8 @@ nntpconnect(const struct server *upstream)
 
 	    reply = nntpreply(upstream);
 	    if (reply == 200 || reply == 201) {
-		syslog(LOG_INFO, "%s: connected to %s:%d, reply: %d",
-			upstream->name,
-			inet_ntoa(s_in.sin_addr), (int)ntohs(s_in.sin_port),
-			reply);
+		syslog(LOG_INFO, "%s: connected to %s:%hd, reply: %d",
+			upstream->name, buf, ntohs(sp->s_port), reply);
 		line = lastreply();
 		if (line == NULL) {
 		    ln_log(LNLOG_SWARNING, LNLOG_CTOP,
@@ -442,6 +485,10 @@ nntpconnect(const struct server *upstream)
 			    "Emulating STAT with HEAD at the expense of bandwidth.",
 			    upstream->name, line);
 		}
+#ifdef HAVE_IPV6
+		if (ai)
+		    freeaddrinfo(ai);
+#endif
 		return reply;
 	    } else { /* reply not 200 and not 201 */
 		char *ll = lastreply();
@@ -450,32 +497,44 @@ nntpconnect(const struct server *upstream)
 		nntpquit();
 	    }
 	} /* end of IP-addresses for loop */
+#ifdef HAVE_IPV6
+	if (!aii)
+#else
 	if (!(int *)(hp->h_addr_list)[i])
+#endif
 	    ln_log(LNLOG_SNOTICE, LNLOG_CTOP,
 		    "%s: address list exhausted without establishing connection.",
 		    upstream->name);
+#ifdef HAVE_IPV6
+	if (ai)
+	    freeaddrinfo(ai);
+#endif
     } else {
-	/* gethostbyname returned NULL */
-	const char *e;
+	/* gethostbyname or getaddrinfo returned error */
+	const char *er;
+#ifdef HAVE_IPV6
+	er = gai_strerror(e);
+#else
 	switch(h_errno) {
 	    case HOST_NOT_FOUND:
-		e = "No such host.";
+		er = "No such host.";
 		break;
 	    case NO_DATA:
-		e = "Name exists in DNS, but has no associated address (\"A\"-type DNS resource record).";
+		er = "Name exists in DNS, but has no associated address (\"A\"-type DNS resource record).";
 		break;
 	    case NO_RECOVERY:
-		e = "Unexpected permanent server failure.";
+		er = "Unexpected permanent server failure.";
 		break;
 	    case TRY_AGAIN:
-		e = "Temporary DNS error, please try again later.";
+		er = "Temporary DNS error, please try again later.";
 		break;
 	    default:
-		e = "Unknown h_errno value.";
+		er = "Unknown h_errno value.";
 		break;
 	}
+#endif
 	ln_log(LNLOG_SWARNING, LNLOG_CTOP,
-		"warning: %s: cannot resolve host name: %s", upstream->name, e);
+		"warning: %s: cannot resolve host name: %s", upstream->name, er);
     }
     return FALSE;
 }				/* end of connect function */
