@@ -363,8 +363,8 @@ dogroup(/*@null@*/ struct newsgroup *g, const char *name, int expdays)
 					|| link(de->d_name, t)))
 			    {
 				    ln_log(LNLOG_SERR, LNLOG_CGROUP,
-					   "%s: relink of %s <-> %s failed: %m (%s)",
-					   name, p, de->d_name, t);
+					   "%s: relink of %s <-> %s failed: %s (%s)",
+					   name, p, de->d_name, strerror(errno), t);
 			    } else {
 				ln_log(LNLOG_SINFO, LNLOG_CARTICLE,
 					"%s: relinked message %s <-> %s", name, p, de->d_name);
@@ -422,11 +422,11 @@ dogroup(/*@null@*/ struct newsgroup *g, const char *name, int expdays)
 	g->last = last;
 
     if (!quiet)
-	printf("%s: %d articles deleted (%lu duplicates), %d kept\n",
-		name, deleted, dupli, kept);
+	printf("%s: %d article%s deleted (%lu duplicate%s), %d kept\n",
+		name, deleted, PLURAL(deleted), dupli, PLURAL(dupli), kept);
     syslog(LOG_INFO,
-	    "%s: %d articles deleted (%lu duplicates), %d kept",
-	    name, deleted, dupli, kept);
+	    "%s: %d article%s deleted (%lu duplicate%s), %d kept",
+	    name, deleted, PLURAL(deleted), dupli, PLURAL(dupli), kept);
 
     if (!kept) {
 	if (unlink(".overview") < 0)
@@ -560,7 +560,7 @@ readmids(void)
 static int
 cleanmids(void)
 {
-    int n;
+    int n, rc = 0;
     mastr *s = mastr_new(256);
 
     for (n = 0; n < 1000; n++) {
@@ -569,9 +569,10 @@ cleanmids(void)
 	mastr_clear(s);
 	mastr_vcat(s, spooldir, "/message.id/", buf, "/", MIDSFILE, NULL);
 	if (log_unlink(mastr_str(s), 1))
-	    return 1;
+	    rc = 1;
     }
-    return 0;
+    mastr_delete(s);
+    return rc;
 }
 
 static void
@@ -662,8 +663,8 @@ expiremsgid(void)
 	puts("Done.");
 
     if (!quiet)
-	printf("total: %d articles deleted, %d kept\n", deleted, kept);
-    syslog(LOG_INFO, "%d articles deleted, %d kept", deleted, kept);
+	printf("total: %d article%s deleted, %d kept\n", deleted, PLURAL(deleted), kept);
+    syslog(LOG_INFO, "%d article%s deleted, %d kept", deleted, PLURAL(deleted), kept);
 }
 
 
